/*-
 * Copyright (c) 2000 Valentin Nechayev <netch@netch.kiev.ua>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: knxcon.c,v 1.2 2000/09/16 08:39:10 netch Exp $
 */

#include <sys/param.h>
#include "pcvt_ioctl.h"
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#ifndef lint
static const char RcsID[] = "$Id: knxcon.c,v 1.2 2000/09/16 08:39:10 netch Exp $";
#endif

static int Fd = -1;

static void BigClear( void );
static void Usage( void );
static void PrintStatus( void );
static void SetDebug( int );
static void SetLayout( const char* );

int
main( int argc, char *argv[] )
{
   int opt;
   int CallBigClear = 0;
   int DoPrintStatus = 0;
   int flag_setdebug = -1;
   const char* ConsPath = NULL;
   const char* LPath = NULL;
   if( argc <= 1 )
      Usage();
   while( ( opt = getopt( argc, argv, "Cd:f:l:S" ) ) != -1 )
   {
      switch( opt )
      {
         case 'C':
            CallBigClear = 1;
         break;
         case 'd':
            flag_setdebug = strtol( optarg, NULL, 0 );
         break;
	 case 'S':
	    DoPrintStatus = 1;
	 break;
         case 'f':
            ConsPath = optarg;
         break;
         case 'l':
            LPath = optarg;
         break;
         default:
            Usage();
         break;
      }
   }
   if( ConsPath ) {
      Fd = open( ConsPath, O_RDWR, 0 );
      if( Fd == -1 ) {
         perror( "knxcon: open()" );
         exit( 1 );
      }
   }
   else
      Fd = 0; /* XXX use stdin */
   /**** print status */
   if( flag_setdebug >= 0 )
      SetDebug( flag_setdebug );
   if( CallBigClear )
      BigClear();
   if (DoPrintStatus)
      PrintStatus();
   if( LPath != NULL )
      SetLayout( LPath );
   return 0;
}

static void
Usage( void )
{
   fprintf( stderr, "knxcon: usage\n" );
   exit( 1 );
}

static int Weq(const char* v, int n, const char* p)
{
return strlen (p) == n && !strncmp(v,p,n);
};

static void
PrintStatus( void )
{
   int iparm;
   u_short keynum;
   kbd_nx_keydef kd;

   if( ioctl( Fd, KBDNXGMODE, &iparm ) < 0 )
      perror( "knxcon: ioctl()" );
   else {
      const char* mn = NULL;
      if( iparm == 0 )
         mn = "terminal";
      else if( iparm == 1 )
         mn = "keycodes";
      else if( iparm == 2 )
         mn = "scancodes";
      else
         mn = "??what??";
      printf( "Current keyboard mode: %s(%d)\n", mn, iparm );
   }
   for( keynum = 0; keynum < 128; keynum++ ) {
      memset( &kd, 0, sizeof kd );
      kd.kd_keynum = keynum;
      if( ioctl( Fd, KBDNXGKEYDEF, &kd ) < 0 ) {
         fprintf( stderr, "knxcon: ioctl(KBDNXGKEYDEF,%u): %s\n",
               (unsigned) keynum, strerror( errno ) );
         continue;
      }
      if( kd.kd_flags & KDF_PRESENT )
         printf( "Key %u is translated in pcvt_nx\n", (unsigned) keynum );
   }
}

static void
BigClear( void )
{
   int Bad = 0;
   int safemode = 0;
   if( ioctl( Fd, KBDNXSMODE, &safemode ) < 0 ) {
      perror( "knxcon: ioctl()" );
      Bad = 1;
   }
   if( ioctl( Fd, KBDNXSCLEARKEYS, NULL ) < 0 ) {
      perror( "knxcon: ioctl()" );
      Bad = 1;
   }
   if( Bad )
      exit( 1 );
}

static void
SetDebug( int L )
{
   if( ioctl( Fd, KBDNXSDEBUG,  &L ) < 0 ) {
      perror( "knxcon: ioctl(KBDNXSDEBUG)" );
   }
}

static void
SetLayout( const char* LPath )
{
   FILE* f;
   char Buf[ 1024 ];
   f = fopen( LPath, "r" );
   if( !f )  { perror( "fopen()" ); exit( 1 ); }
   while( !ferror( f ) && !feof( f ) ) {
      char *p, *p1, *p2, *p3, *p4, *p5;
      int n1, n2, n3, n4;
      if( !fgets( Buf, sizeof Buf, f ) )
         break;
      p = strchr( Buf, '\n' );
      if( !p ) {
         fprintf( stderr, "kxncon: too long line\n" );
         exit( 1 );
      }
      *p = 0;
      /* Split line */
      p = Buf;
      while( *p && isspace( *p ) )
         p++;
      if( !*p || !isalnum( *p ) )
         continue;
      p2 = p;
      while( *p2 && !isspace( *p2 ) )
         p2++;
      n1 = p2 - p;
      while( *p2 && isspace( *p2 ) )
         p2++;
      p3 = p2;
      while( *p3 && !isspace( *p3 ) )
         p3++;
      n2 = p3 - p2;
      while( *p3 && isspace( *p3 ) )
         p3++;
      p4 = p3;
      while( *p4 && !isspace( *p4 ) )
         p4++;
      n3 = p4 - p3;
      while( *p4 && isspace( *p4 ) )
         p4++;
      p5 = p4;
      while( *p5 && !isspace( *p5 ) )
         p5++;
      n4 = p5 - p4;
      if( Weq( p, n1, "keydef" ) ) {
         kbd_nx_keydef kd;
         memset( &kd, 0, sizeof kd );
         kd.kd_keynum = strtoul( p2, NULL, 0 );
         kd.kd_flags = strtoul( p3, NULL, 0 );
         kd.kd_smask = strtoul( p4, NULL, 0 );
         kd.kd_nvalues = strtoul( p5, NULL, 0 );
         kd.kd_values = NULL; /* we don't define them here */
         if( ioctl( Fd, KBDNXSKEYDEF, &kd ) < 0 ) {
            perror( "ioctl(KBDNXSKEYDEF)" );
            exit( 1 );
         }
      }
      else if( Weq( p, n1, "1keyvalue" ) ) {
         register u_short keynum = strtoul( p2, NULL, 0 );
         register u_short kvind = strtoul( p3, NULL, 0 );
         kbd_nx_keyvalue kvs[ KBD_NX_MAXVALUES ];
         kbd_nx_keydef kd;
         memset( kvs, 0, sizeof kvs );
         memset( &kd, 0, sizeof kd );
         kd.kd_keynum = keynum;
         kd.kd_values = kvs;
         kd.kd_nvalues = KBD_NX_MAXVALUES;
         if( ioctl( Fd, KBDNXGKEYDEF, &kd ) < 0 ) {
            perror( "ioctl(KBDNXGKEYDEF)" );
            exit( 1 );
         }
         kvs[kvind].kv_flags = 0;
         kvs[kvind].kv_value = strtoul( p4, NULL, 0 );
         kd.kd_keynum = keynum;
         kd.kd_values = kvs;
         kd.kd_nvalues = KBD_NX_MAXVALUES;
         if( ioctl( Fd, KBDNXSKEYDEF, &kd ) < 0 ) {
            perror( "ioctl(KBDNXSKEYDEF)" );
            exit( 1 );
         }
      }
      else {
         fprintf( stderr, "knxcon: unknown layout command `%*.*s'\n",
               n1, n1, p );
         exit( 1 );
      }
   }
   if( ferror( f ) ) {
      perror( "ferror(f)" );
      exit( 1 );
   }
}
